/* debug.c -  simple debug subsystem */
/* Sun 17 May 2020 02:27:32 PM EDT */
/* $Id: debug.c,v 1.24 2024/03/27 18:11:32 malakai Exp $ */

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <stdarg.h>
#include <ctype.h>
#include <errno.h>

#include "structs.h"
#include "utils.h"
#include "comm.h"
#include "handler.h"
#include "scnprintf.h"

#include "debug.h"

/* Should debug mode be loaded by lo_debug_init() and stored by save_debug() when changed? */
#define SAVE_MODE_ENABLED 1
#define DEBUG_FACILITY_FILENAME "init_debug_mode"

/* if you want a facility on at boot time, you can set it here. */
unsigned int global_debug_facility = 0;

char *facility_names[] = {
	"input",
	"mobact",
	"skilluse",
	"skillfail",
	"fight",
	"mobflags",
	"tmap",
	"db",
	"break",
	"testing",
	"pather",
	"exploration",
	"telnet",
	"knowledge",
	"water",
	"journal",
	"crafting",
	"itemdam",
	"magic",
	"pulse",
	"event",
	"gmcp",
	"\n"
};

/* Functionally, its a fancy wrapper around printf! */
void lo_Debug(int facility, const char *caller, char *str, ...)
{

	va_list ap;
	char vbuf[MAX_STRING_LENGTH];
	char *v,*eov;

	if(((facility) & global_debug_facility) == 0) { 
		return;
	}

	v = vbuf;
	eov = vbuf + sizeof(vbuf)-1;

	v += scnprintf(v,eov-v,"%s(): ",caller);

	va_start(ap,str);
	vscnprintf(v,eov-v,str,ap);
	va_end(ap);

	lo_log("%s",vbuf);

	if(global_debug_facility & DEBUG_BREAK) {
		lo_breakpoint(facility);
	}
}

/* This is called at the end of lo_Debug() if DEBUG_BREAK is enabled, so that
 * you can easily break on a debug print in gdb with "break lo_breakpoint".
 * The code is just a number passed in with the call in case you want to make a
 * gdb conditional break on that.  You can use this function directly in code
 * as you develop if you want, but don't leave it in production outside of
 * lo_Debug().  Its probably better to use as lo_debug(DEBUG_BREAK,"") as a
 * simple development breakpoint.  */

int lo_breakpoint(int code) {
	static int count = 0;
	count++;
	return(count);
}


/* There's not really much to init, in fact you don't even have to call this.
 * All it really does is load up a saved debug level from the file so that you
 * can debug things at boot up time. Call it after the game has cwd'd to the
 * data directory.*/
void lo_debug_init(void) {

	FILE *in;

	if (!SAVE_MODE_ENABLED) return;

	if((in = fopen(DEBUG_FACILITY_FILENAME,"r"))) {
		fscanf(in,"%x",&global_debug_facility);
		fclose(in);
	}
	if(global_debug_facility != 0) {
		lo_log("**** Loaded debug init level %#x ****",global_debug_facility);
	} 
	return;

}

/* Write the debug mode out to a persistent file. */
void save_debug(void) {
	FILE *out;

	if (!SAVE_MODE_ENABLED) return;

	if( !(out = fopen(DEBUG_FACILITY_FILENAME,"w"))) {
		return;
	} 
	fprintf(out,"%#x\n",global_debug_facility);
	fclose(out);
	return;

}

void do_debug(struct char_data *ch, char *argument, int cmd) {

	char buf[MAX_STRING_LENGTH];
	char *s,*eos;

	if(*argument) {
		if(isnameoverlap(argument,"off")) {
			global_debug_facility =0;
			save_debug();
		} else {
			int changed = 0;
			for(int i=0;i<DEBUG_MAX;i++) {
				if( isnameoverlap(argument,facility_names[i]) ) { 
					global_debug_facility ^= (1<<i);
					save_debug();
					changed++;
				}
			}
			if(!changed) {
				s=buf;
				eos = s+sizeof(buf)-1;
				s += scnprintf(s,eos-s,"Debug facilites are: ");
				for(int i=0;i<DEBUG_MAX;i++) {
					s += scnprintf(s,eos-s,"%s ",facility_names[i]);
				}
				act(buf,FALSE,ch,NULL,NULL,TO_INFO);
			}
		}
	} else {
		act("Use 'debug off' to disable, 'debug list' to show facilities.",FALSE,ch,NULL,NULL,TO_INFO);
	}

	s=buf;
	eos = s+sizeof(buf)-1;

	s += scnprintf(s,eos-s,"Debug enabled %#x: ",global_debug_facility);
	for(int i=0;i<DEBUG_MAX;i++) {
		if( global_debug_facility & (1<<i) ) {
			s += scnprintf(s,eos-s,"%s ",facility_names[i]);
		}
	}
	act(buf,FALSE,ch,NULL,NULL,TO_INFO);

}

void lo_perror(const char *s) {
	lo_log("perror(\"%s\"): %s",s,strerror(errno));
}
